package com.huimv.production.task;

import com.huimv.production.domain.*;
import com.huimv.production.repo.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import javax.transaction.Transactional;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * 定时任务
 * </p>
 *
 * @author yinhao
 * @date 2021/5/28 10:34
 */
@Configuration
@EnableScheduling
public class WarningParameterTask {

    @Autowired
    private ConfigEntityRepo configEntityRepo;

    //本地参数
    @Autowired
    private IndexParameter2EntityRepo indexParameter2EntityRepo;
    //erp 参数
    @Autowired
    private IndexParameterEntityRepo indexParameterEntityRepo;

    @Autowired
    private WarningInfoEntityRepo warningInfoEntityRepo;

    @Autowired
    private WarningParameterEntityRepo warningParameterEntityRepo;

    @Transactional(rollbackOn = Throwable.class)
    @Scheduled(cron = "0 0 0/12 * * ?")
//    @Scheduled(cron = "0 41 17 * * ?")
    public void warningTask() {
        List<ConfigEntity> all = configEntityRepo.findAll();
        ConfigEntity configEntity = all.get(0);
        String propertyValue = configEntity.getValue();

        List<WarningParameter> warningParameterList = warningParameterEntityRepo.findAll();

        //当前时间
        Date date = new Date();

        if ("true".equals(propertyValue)) {
            List<IndexParameter2Entity> localList = indexParameter2EntityRepo.findAll();
            List<IndexParameter2Entity> collect = localList.stream()
                    .filter(localEntity -> warningParameterList.stream()
                            .anyMatch(w -> w.getParameterName().equals(localEntity.getName())))
                    .collect(Collectors.toList());

            List<IndexParameter2Entity> collect1 = collect.stream().filter(s -> warningParameterList.stream()
                    .anyMatch(warningParameter -> warningParameter.getParameterName().equals(s.getName()) && warningParameter.getThresholdValue() >= Double.parseDouble(s.getData())))
                    .collect(Collectors.toList());


            for (IndexParameter2Entity indexParameter2Entity : collect1) {
                WarningInfoEntity warningInfoEntity = new WarningInfoEntity();
                warningInfoEntity.setUploadTime(new Date());
                warningInfoEntity.setWarningType(3);
                String type = "本地数据的 " + indexParameter2Entity.getName() + " 参数小于阈值";
                warningInfoEntity.setType(type);
                warningInfoEntityRepo.updatePreviousShowStatusByType(type);
                warningInfoEntityRepo.save(warningInfoEntity);
            }

            collect.removeAll(collect1);

            List<WarningInfoEntity> list = new ArrayList<>();
            for (IndexParameter2Entity indexParameter2Entity : collect) {
                List<WarningInfoEntity> warningInfoEntityList = warningInfoEntityRepo.findDataTypeLikeLeNow("本地数据的 " + indexParameter2Entity.getName(), date);
                list.addAll(warningInfoEntityList);
            }
            for (WarningInfoEntity warningInfoEntity : list) {
                warningInfoEntity.setShowStatus(1);
                warningInfoEntityRepo.saveAndFlush(warningInfoEntity);
            }
        }

        if ("false".equals(propertyValue)) {
            List<IndexParameterEntity> localList = indexParameterEntityRepo.findAll();
            List<IndexParameterEntity> collect = localList.stream()
                    .filter(localEntity -> warningParameterList.stream()
                            .anyMatch(w -> w.getParameterName().equals(localEntity.getName())))
                    .collect(Collectors.toList());

            List<IndexParameterEntity> collect1 = collect.stream().filter(s -> warningParameterList.stream()
                    .anyMatch(warningParameter -> warningParameter.getParameterName().equals(s.getName()) && warningParameter.getThresholdValue() >= Double.parseDouble(s.getData())))
                    .collect(Collectors.toList());

            for (IndexParameterEntity indexParameter2Entity : collect1) {
                WarningInfoEntity warningInfoEntity = new WarningInfoEntity();
                warningInfoEntity.setUploadTime(new Date());
                warningInfoEntity.setWarningType(3);
                String type = "ERP数据的 " + indexParameter2Entity.getName() + " 参数小于阈值!";
                warningInfoEntity.setType(type);
                warningInfoEntityRepo.updatePreviousShowStatusByType(type);
                warningInfoEntityRepo.save(warningInfoEntity);
            }

            collect.removeAll(collect1);

            List<WarningInfoEntity> list = new ArrayList<>();
            for (IndexParameterEntity indexParameterEntity : collect) {
                List<WarningInfoEntity> warningInfoEntityList = warningInfoEntityRepo.findDataTypeLikeLeNow("ERP数据的 " + indexParameterEntity.getName(), date);
                list.addAll(warningInfoEntityList);
            }
            for (WarningInfoEntity warningInfoEntity : list) {
                warningInfoEntity.setShowStatus(1);
                warningInfoEntityRepo.saveAndFlush(warningInfoEntity);
            }

        }
    }
}